@@ -4,37 +4,33 @@ use std::path::Path;
 use anyhow::Result;
 use log::error;
 
-use ruff_linter::source_kind::SourceKind;
+use ruff_linter::source_kind::{SourceError, SourceKind};
 use ruff_python_ast::{PySourceType, SourceType};
-use ruff_workspace::resolver::{match_exclusion, python_file_at_path, Resolver};
 use ruff_workspace::FormatterSettings;
+use ruff_workspace::resolver::{PyprojectConfig, Resolver, match_exclusion, python_file_at_path};
 
-use crate::args::{CliOverrides, FormatArguments, FormatRange};
-use crate::commands::format::{
-    format_source, warn_incompatible_formatter_settings, FormatCommandError, FormatMode,
-    FormatResult, FormattedSource,
-};
-use crate::resolve::resolve;
-use crate::stdin::{parrot_stdin, read_from_stdin};
 use crate::ExitStatus;
+use crate::args::{ConfigArguments, FormatArguments, FormatRange};
+use crate::commands::format::{
+    FormatCommandError, FormatMode, FormatResult, FormattedSource, format_source,
+    warn_incompatible_formatter_settings,
+};
+use crate::stdin::{parrot_stdin, read_from_stdin};
 
 /// Run the formatter over a single file, read from `stdin`.
-pub(crate) fn format_stdin(cli: &FormatArguments, overrides: &CliOverrides) -> Result<ExitStatus> {
-    let pyproject_config = resolve(
-        cli.isolated,
-        cli.config.as_deref(),
-        overrides,
-        cli.stdin_filename.as_deref(),
-    )?;
-
-    let mut resolver = Resolver::new(&pyproject_config);
+pub(crate) fn format_stdin(
+    cli: &FormatArguments,
+    config_arguments: &ConfigArguments,
+    pyproject_config: &PyprojectConfig,
+) -> Result<ExitStatus> {
+    let mut resolver = Resolver::new(pyproject_config);
     warn_incompatible_formatter_settings(&resolver);
 
     let mode = FormatMode::from_cli(cli);
 
     if resolver.force_exclude() {
         if let Some(filename) = cli.stdin_filename.as_deref() {
-            if !python_file_at_path(filename, &mut resolver, overrides)? {
+            if !python_file_at_path(filename, &mut resolver, config_arguments)? {
                 if mode.is_write() {
                     parrot_stdin()?;
                 }
@@ -57,7 +53,7 @@ pub(crate) fn format_stdin(cli: &FormatArguments, overrides: &CliOverrides) -> R
 
     let source_type = match path.and_then(|path| settings.extension.get(path)) {
         None => match path.map(SourceType::from).unwrap_or_default() {
-            SourceType::Python(source_type) => source_type,
+            source_type @ (SourceType::Python(_) | SourceType::Markdown) => source_type,
             SourceType::Toml(_) => {
                 if mode.is_write() {
                     parrot_stdin()?;
@@ -65,7 +61,7 @@ pub(crate) fn format_stdin(cli: &FormatArguments, overrides: &CliOverrides) -> R
                 return Ok(ExitStatus::Success);
             }
         },
-        Some(language) => PySourceType::from(language),
+        Some(language) => SourceType::Python(PySourceType::from(language)),
     };
 
     // Format the file.
@@ -92,7 +88,7 @@ fn format_source_code(
     path: Option<&Path>,
     range: Option<FormatRange>,
     settings: &FormatterSettings,
-    source_type: PySourceType,
+    source_type: SourceType,
     mode: FormatMode,
 ) -> Result<FormatResult, FormatCommandError> {
     // Read the source from stdin.
@@ -108,7 +104,7 @@ fn format_source_code(
     };
 
     // Format the source.
-    let formatted = format_source(&source_kind, source_type, path, settings, range)?;
+    let formatted = format_source(&source_kind, path, settings, range)?;
 
     match &formatted {
         FormattedSource::Formatted(formatted) => match mode {
@@ -120,9 +116,15 @@ fn format_source_code(
             }
             FormatMode::Check => {}
             FormatMode::Diff => {
-                source_kind
-                    .diff(formatted, path, &mut stdout().lock())
-                    .map_err(|err| FormatCommandError::Diff(path.map(Path::to_path_buf), err))?;
+                use std::io::Write;
+                write!(
+                    &mut stdout().lock(),
+                    "{}",
+                    source_kind.diff(formatted, path).unwrap()
+                )
+                .map_err(|err| {
+                    FormatCommandError::Write(path.map(Path::to_path_buf), SourceError::Io(err))
+                })?;
             }
         },
         FormattedSource::Unchanged => {
