@@ -39,6 +39,7 @@ use crate::{AlwaysFixableViolation, Fix};
 /// ## Options
 /// - `lint.isort.required-imports`
 #[derive(ViolationMetadata)]
+#[violation_metadata(stable_since = "v0.0.218")]
 pub(crate) struct MissingRequiredImport(pub String);
 
 impl AlwaysFixableViolation for MissingRequiredImport {
@@ -58,7 +59,12 @@ impl AlwaysFixableViolation for MissingRequiredImport {
 fn includes_import(stmt: &Stmt, target: &NameImport) -> bool {
     match target {
         NameImport::Import(target) => {
-            let Stmt::Import(ast::StmtImport { names, range: _ }) = &stmt else {
+            let Stmt::Import(ast::StmtImport {
+                names,
+                range: _,
+                node_index: _,
+            }) = &stmt
+            else {
                 return false;
             };
             names.iter().any(|alias| {
@@ -72,6 +78,7 @@ fn includes_import(stmt: &Stmt, target: &NameImport) -> bool {
                 names,
                 level,
                 range: _,
+                node_index: _,
             }) = &stmt
             else {
                 return false;
@@ -119,7 +126,8 @@ fn add_required_import(
         TextRange::default(),
     );
     diagnostic.set_fix(Fix::safe_edit(
-        Importer::new(parsed, locator, stylist).add_import(required_import, TextSize::default()),
+        Importer::new(parsed, locator.contents(), stylist)
+            .add_import(required_import, TextSize::default()),
     ));
 }
 
@@ -132,7 +140,7 @@ pub(crate) fn add_required_imports(
     source_type: PySourceType,
     context: &LintContext,
 ) {
-    for required_import in &settings.isort.required_imports {
+    for required_import in settings.isort.required_imports.iter().rev() {
         add_required_import(
             required_import,
             parsed,
