@@ -1,3 +1,4 @@
+use ruff_diagnostics::Applicability;
 use ruff_python_ast::{self as ast, Arguments, Decorator, Expr, Keyword};
 use ruff_text_size::{Ranged, TextRange};
 
@@ -38,9 +39,13 @@ use crate::{AlwaysFixableViolation, Edit, Fix};
 /// ## Options
 /// - `target-version`
 ///
+/// ## Fix safety
+/// This rule's fix is marked as safe, unless the expression contains comments.
+///
 /// ## References
 /// - [Python documentation: `@functools.cache`](https://docs.python.org/3/library/functools.html#functools.cache)
 #[derive(ViolationMetadata)]
+#[violation_metadata(stable_since = "v0.0.225")]
 pub(crate) struct LRUCacheWithMaxsizeNone;
 
 impl AlwaysFixableViolation for LRUCacheWithMaxsizeNone {
@@ -102,7 +107,21 @@ pub(crate) fn lru_cache_with_maxsize_none(checker: &Checker, decorator_list: &[D
                     )?;
                     let reference_edit =
                         Edit::range_replacement(binding, decorator.expression.range());
-                    Ok(Fix::safe_edits(import_edit, [reference_edit]))
+
+                    let applicability = if checker
+                        .comment_ranges()
+                        .intersects(decorator.expression.range())
+                    {
+                        Applicability::Unsafe
+                    } else {
+                        Applicability::Safe
+                    };
+
+                    Ok(Fix::applicable_edits(
+                        import_edit,
+                        [reference_edit],
+                        applicability,
+                    ))
                 });
             }
         }
