@@ -12,6 +12,7 @@ pub use python_version::*;
 pub mod comparable;
 pub mod docstrings;
 mod expression;
+pub mod find_node;
 mod generated;
 pub mod helpers;
 pub mod identifier;
@@ -29,6 +30,7 @@ pub mod statement_visitor;
 pub mod stmt_if;
 pub mod str;
 pub mod str_prefix;
+pub mod token;
 pub mod traversal;
 pub mod types;
 pub mod visitor;
@@ -41,6 +43,8 @@ pub enum SourceType {
     Python(PySourceType),
     /// The file contains TOML.
     Toml(TomlSourceType),
+    /// The file contains Markdown.
+    Markdown,
 }
 
 impl Default for SourceType {
@@ -57,6 +61,7 @@ impl<P: AsRef<Path>> From<P> for SourceType {
             Some(filename) if filename == "poetry.lock" => Self::Toml(TomlSourceType::Poetry),
             _ => match path.as_ref().extension() {
                 Some(ext) if ext == "toml" => Self::Toml(TomlSourceType::Unrecognized),
+                Some(ext) if ext == "md" => Self::Markdown,
                 _ => Self::Python(PySourceType::from(path)),
             },
         }
@@ -78,7 +83,9 @@ pub enum TomlSourceType {
 #[derive(Clone, Copy, Debug, Default, PartialEq, Eq, Hash)]
 #[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
 pub enum PySourceType {
-    /// The source is a Python file (`.py`).
+    /// The source is a Python file (`.py`, `.pyw`).
+    /// Note: `.pyw` files contain Python code, but do not represent importable namespaces.
+    /// Consider adding a separate source type later if combining the two causes issues.
     #[default]
     Python,
     /// The source is a Python stub file (`.pyi`).
@@ -100,6 +107,7 @@ impl PySourceType {
         let ty = match extension {
             "py" => Self::Python,
             "pyi" => Self::Stub,
+            "pyw" => Self::Python,
             "ipynb" => Self::Ipynb,
             _ => return None,
         };
